# Copyright 2021 Google LLC.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from absl.testing import absltest

import math

from temporian.core.data.node import input_node
from temporian.core.operators.until_next import until_next


class UntilNextOperatorTest(absltest.TestCase):
    def setUp(self):
        pass

    def test_basic(self):
        node = input_node([])
        sampling = input_node([])
        _ = until_next(input=node, sampling=sampling, timeout=5)

    def test_timeout_negative(self):
        node = input_node([])
        sampling = input_node([])

        with self.assertRaisesRegex(
            ValueError, "A duration should be a strictly"
        ):
            _ = until_next(input=node, sampling=sampling, timeout=-5)

    def test_timeout_non_finite(self):
        node = input_node([])
        sampling = input_node([])

        with self.assertRaisesRegex(
            ValueError, "A duration should be a strictly"
        ):
            _ = until_next(input=node, sampling=sampling, timeout=math.nan)


if __name__ == "__main__":
    absltest.main()
